import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { InjectRepository } from "@nestjs/typeorm";
import { Constants } from "src/utils/constants";
import { ResponseUtils } from "src/utils/response.utils";
import { DoSpacesService } from "../SpacesModule/SpacesService/doSpacesService";
import { ProductReviewRepository } from "./product-review.repository";
import { CreateProductReviewDto } from "./dto/create-product-review.dto";
import { ProductReview } from "./entities/product-review.entity";

@Injectable()
export class ProductReviewService {
    constructor(
      @InjectRepository(ProductReviewRepository)
      private readonly repository: ProductReviewRepository,
      private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
      dto: CreateProductReviewDto
    ): Promise<ProductReview | Error> {
        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
              Constants.CREATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(201, "Data created successfully", "data", data);
    }

    async findAll(): Promise<ProductReview[] | Error> {
        const data = await this.repository.findByFilterQuery({});
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", data);
    }

    async findOne(id: string): Promise<ProductReview[] | Error> {
        const data = await this.repository.findByFilterQuery({ productId: id });
        const reviewCount = Object.keys(data).length != 0 ? Object.keys(data).length : 0;
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        let ratingAvarage = 0;
        if (Object.keys(data).length > 0) {
            let count = 0;
            for (let i = 0; i < reviewCount; i++) {
                count += parseInt(data[i].rating);
            }
            ratingAvarage = Math.round(count / reviewCount);
        }

        const response = {
            data: data,
            reviewCount: reviewCount,
            ratingAvarage: ratingAvarage
        };
        return ResponseUtils.successResponseHandler(200, "Data fetched successfully", "data", response);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.buildDeletedData(data, 200, "Data deleted successfully");
    }
}
