import { EntityRepository, getConnection, Repository } from "typeorm";
import { CreateQuizDto } from "./dto/create-quiz.dto";
import { UpdateQuizDto } from "./dto/update-quiz.dto";
import { Quiz } from "./entities/quiz.entity";

@EntityRepository(Quiz)
export class QuizRepository extends Repository<Quiz> {
    async createEntity(dto: CreateQuizDto): Promise<Quiz | Error> {
        try {
            const {
                question,
                answer,
                optionA,
                optionB,
                optionC,
                optionD
            } = dto;
            const entity = this.create({
                question,
                answer,
                optionA,
                optionB,
                optionC,
                optionD
            });
            await this.save(entity);
            return entity;
        } catch (e) {
            return e;
        }
    }

    async findOneEntity(id: string): Promise<Quiz | Error> {
        try {
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async updateEntity(id: string, dto: UpdateQuizDto): Promise<Quiz | Error> {
        try {
            await this.update(id, {
                question: dto.question,
                answer: dto.answer,
                optionA: dto.optionA,
                optionB: dto.optionB,
                optionC: dto.optionC,
                optionD: dto.optionD
            });
            return await this.findOne(id);
        } catch (e) {
            return e;
        }
    }

    async findByFilterQuery(query: any): Promise<Quiz[] | Error> {
        try {
            return await this.find({
                where: { ...query, isDeleted: false }
            });
        } catch (e) {
            return e;
        }
    }

    async deleteEntity(id: string): Promise<boolean | Error> {
        try {
            const updateResult = await getConnection()
              .createQueryBuilder()
              .update(Quiz)
              .set({ isDeleted: true })
              .where("id = :id", { id: id })
              .execute();
            if (updateResult.affected > 0) {
                return true;
            }
            return false;
        } catch (e) {
            return e;
        }
    }
}